from django.db.models import Model, DateField, CharField, ForeignKey, CASCADE


class AbstractCreatedModel(Model):
    date_created = DateField(verbose_name='Date created', auto_now_add=True, editable=False)

    class Meta:
        abstract = True
        ordering = ('-date_created',)


class AbstractDateModel(AbstractCreatedModel):
    date = DateField(verbose_name='Date')

    class Meta:
        abstract = True

    def __str__(self):
        return f'ID: {self.id} | Date: {self.date}'


class Holidays(AbstractDateModel):
    class Meta:
        verbose_name = 'Holiday'
        verbose_name_plural = 'Holidays'


class ExcludedWeekends(AbstractDateModel):
    class Meta:
        verbose_name = 'Excluded weekend'
        verbose_name_plural = 'Excluded weekends'


class CommonWorkingWeekSettings(AbstractCreatedModel):
    class Meta:
        verbose_name = 'Common working week setting'
        verbose_name_plural = 'Common working week settings'

    def __str__(self) -> str:
        return f'Common working week setting | ID: {self.id}'


class Days(Model):
    AVAILABLE_DAYS = ('Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday')

    setting = ForeignKey(CommonWorkingWeekSettings, on_delete=CASCADE, verbose_name='Setting', related_name='days')
    name = CharField(verbose_name='Day name', max_length=9)

    class Meta:
        verbose_name = 'Week day'
        verbose_name_plural = 'Week days'

    def __str__(self):
        return f'ID: {self.id} | Name: {self.name} | Setting: {self.setting}'


from django.db import migrations, models
import django.db.models.deletion
from django.db.transaction import atomic

from django_weekend.models import Days, CommonWorkingWeekSettings


@atomic
def create_setting_and_fill_days_names(apps, schema_editor):
    setting_instance = CommonWorkingWeekSettings.objects.create()
    for day_name in Days.AVAILABLE_DAYS:
        obj = Days(setting=setting_instance, name=day_name)
        obj.save()


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CommonWorkingWeekSettings',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_created', models.DateField(auto_now_add=True, verbose_name='Date created')),
            ],
            options={
                'verbose_name': 'Common working week setting',
                'verbose_name_plural': 'Common working week settings',
            },
        ),
        migrations.CreateModel(
            name='ExcludedWeekends',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_created', models.DateField(auto_now_add=True, verbose_name='Date created')),
                ('date', models.DateField(verbose_name='Date')),
            ],
            options={
                'verbose_name': 'Excluded weekend',
                'verbose_name_plural': 'Excluded weekends',
            },
        ),
        migrations.CreateModel(
            name='Holidays',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_created', models.DateField(auto_now_add=True, verbose_name='Date created')),
                ('date', models.DateField(verbose_name='Date')),
            ],
            options={
                'verbose_name': 'Holiday',
                'verbose_name_plural': 'Holidays',
            },
        ),
        migrations.CreateModel(
            name='Days',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=9, verbose_name='Day name')),
                ('setting', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='days', to='django_weekend.commonworkingweeksettings', verbose_name='Setting')),
            ],
            options={
                'verbose_name': 'Week day',
                'verbose_name_plural': 'Week days',
            },
        ),
        migrations.RunPython(create_setting_and_fill_days_names),
    ]



from setuptools import setup, find_packages


def readme():
  with open('README.md', 'r') as f:
    return f.read()


setup(
  name='django_weekend',
  version='0.0.6.3',
  author='Maksim Prilepsky',
  author_email='maksimprilepsky@yandex.ru',
  description='A simple library for tracking weekends and working days',
  long_description=readme(),
  long_description_content_type='text/markdown',
  url='https://github.com/MaksimJames/django_weekend',
  packages=find_packages(),
  install_requires=[],
  classifiers=[
    'Programming Language :: Python :: 3.11',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent'
  ],
  keywords='django calendar settings weekend',
  project_urls={
    'GitHub': 'https://github.com/MaksimJames/django_weekend'
  },
  python_requires='>=3.6'
)


from django import forms
from django.contrib import admin
from django.core.exceptions import ValidationError

from .models import Holidays, ExcludedWeekends, CommonWorkingWeekSettings, Days


class DaysFormAdmin(forms.ModelForm):
    class Meta:
        model = Days
        fields = '__all__'

    def clean(self):
        cleaned_data = super().clean()
        name = cleaned_data.get('name')
        if name and name not in Days.AVAILABLE_DAYS:
            raise ValidationError(f'Not available day name. Available: {", ".join(Days.AVAILABLE_DAYS)}')
        return cleaned_data


class InlineDaysAdmin(admin.TabularInline):
    model = Days
    extra = 1
    can_delete = True
    form = DaysFormAdmin


@admin.register(Holidays)
class HolidaysAdmin(admin.ModelAdmin):
    list_display = ('id', 'date')
    date_hierarchy = 'date'


@admin.register(ExcludedWeekends)
class ExcludedWeekendsAdmin(admin.ModelAdmin):
    list_display = ('id', 'date')
    date_hierarchy = 'date'


@admin.register(CommonWorkingWeekSettings)
class CommonWorkingWeekSettingsAdmin(admin.ModelAdmin):
    inlines = (InlineDaysAdmin,)

    # TODO Если переделывать что бы настройки были для каждого пользователя, то переопределить этот метод,
    # потому что может быть несколько настроек для каждого пользователя
    def has_add_permission(self, request):
        is_has_perms = super().has_add_permission(request)
        if not is_has_perms:
            return False
        return not CommonWorkingWeekSettings.objects.exists()


import datetime
from typing import List

from .models import Holidays, ExcludedWeekends, CommonWorkingWeekSettings


class HolidaysWeekendsService:
    def __init__(self) -> None:
        self.working_week_settings = CommonWorkingWeekSettings.objects.first()
        if not self.working_week_settings:
            raise ValueError('Common working week settings not found. Make sure this is exist.')

    def _is_holiday(self, date: datetime.date) -> bool:
        return Holidays.objects.filter(date=date).exists()

    def _is_excluded_weekend(self, date: datetime.date) -> bool:
        return ExcludedWeekends.objects.filter(date=date).exists()

    def _get_day_name(self, date: str) -> str:
        return date.strftime('%A')

    def _is_weekend(self, day: str) -> bool:
        return not self.working_week_settings.days.filter(name=day).exists()

    def _get_dates_range(self, start_date: str, end_date: str) -> List[str]:
        """
            Generate a list of dates between start_date and end_date.
            Args:
                start_date (str): The start date in the format 'YYYY-MM-DD'.
                end_date (str): The end date in the format 'YYYY-MM-DD'.
            Returns:
                List[str]: A list of dates between start_date and end_date in the format 'YYYY-MM-DD'.
        """
        dates_range = []
        start_date_obj = datetime.datetime.strptime(start_date, '%Y-%m-%d')
        end_date_obj = datetime.datetime.strptime(end_date, '%Y-%m-%d')
        current_date = start_date_obj
        while current_date <= end_date_obj:
            dates_range.append(current_date)
            current_date += datetime.timedelta(days=1)

        return [date.strftime('%Y-%m-%d') for date in dates_range]

    def is_weekend_by_date(self, date: str) -> bool:
        """
            Determines if a given date is a weekend or a holiday.

            Args:
                date (str): The date in the format 'YYYY-MM-DD'.

            Returns:
                bool: True if the date is a weekend or a holiday, False otherwise.
        """
        date_obj = datetime.datetime.strptime(date, '%Y-%m-%d')
        date_name = self._get_day_name(date=date_obj)
        is_weekend = self._is_weekend(day=date_name)
        is_excluded_weekend = self._is_excluded_weekend(date=date_obj)
        is_holiday = self._is_holiday(date=date_obj)
        is_weekend = is_weekend and not is_excluded_weekend

        return is_weekend or is_holiday

    def get_weekends_by_range(self, start_date: str, end_date: str) -> List[str]:
        """
            Returns a list of weekend dates between the given start_date and end_date.

            Args:
                start_date (str): The start date in the format 'YYYY-MM-DD'.
                end_date (str): The end date in the format 'YYYY-MM-DD'.

            Returns:
                List[str]: A list of weekend dates in the format 'YYYY-MM-DD'.
        """
        dates_range = self._get_dates_range(start_date=start_date, end_date=end_date)
        return [date for date in dates_range if self.is_weekend_by_date(date=date)]